/*
 *      Copyright (c) 2016 Samsung Electronics Co., Ltd
 *
 *      Licensed under the Flora License, Version 1.1 (the "License");
 *      you may not use this file except in compliance with the License.
 *      You may obtain a copy of the License at
 *
 *              http://floralicense.org/license/
 *
 *      Unless required by applicable law or agreed to in writing, software
 *      distributed under the License is distributed on an "AS IS" BASIS,
 *      WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *      See the License for the specific language governing permissions and
 *      limitations under the License.
 */

/* exported moonphase */

/**
 * moonphase.js controls moon phase module.
 * It calculates angle for today's moon phase.
 */
var moonphase = (function() {
    var moonphase = {},

        /**
         * Moon phase stamp of next five years.
         * Sets only phase stamp.
         * @type {string[]}
         */
        MOON_PHASE_DATA = [
            // 2016 data set.
            // first phase stamp is "Last Quarter".
            [
                "0102","0110","0116","0124",
                "0201","0208","0215","0222",
                "0301","0309","0315","0323","0331",
                "0407","0414","0422","0430",
                "0506","0513","0521","0529",
                "0605","0612","0620","0627",
                "0704","0712","0719","0726",
                "0802","0810","0818","0825",
                "0901","0909","0916","0923",
                "1001","1009","1016","1022","1030",
                "1107","1114","1121","1129",
                "1207","1214","1221","1229"
            ],

            // 2017 data set.
            // first phase stamp is "First Quarter".
            [
                "0105","0112","0119","0128",
                "0204","0211","0218","0226",
                "0305","0312","0320","0328",
                "0403","0411","0419","0426",
                "0503","0510","0519","0525",
                "0601","0609","0617","0624",
                "0701","0709","0716","0723","0730",
                "0807","0815","0821","0829",
                "0906","0913","0920","0928",
                "1005","1012","1019","1027",
                "1104","1110","1118","1126",
                "1203","1210","1218","1226"
            ],

            // 2018 data set.
            // first phase stamp is "Full Moon".
            [
                "0102","0108","0117","0124","0131",
                "0207","0215","0223",
                "0302","0309","0317","0324","0331",
                "0408","0416","0422","0430",
                "0508","0515","0522","0529",
                "0606","0613","0620","0628",
                "0706","0713","0719","0727",
                "0804","0811","0818","0826",
                "0903","0909","0916","0925",
                "1002","1009","1016","1024","1031",
                "1107","1115","1123","1130",
                "1207","1215","1222","1229"
            ],

            // 2019 data set.
            // first phase stamp is "New Moon".
            [
                "0106","0114","0121","0127",
                "0204","0212","0219","0226",
                "0306","0314","0321","0328",
                "0405","0412","0419","0426",
                "0504","0512","0518","0526",
                "0603","0610","0617","0625",
                "0702","0709","0716","0725",
                "0801","0807","0815","0823","0830",
                "0906","0914","0922","0928",
                "1005","1013","1021","1028",
                "1104","1112","1119","1126",
                "1204","1212","1219","1226"
            ],

            // 2020 data set.
            // first phase stamp is "First Quarter".
            [
                "0103","0110","0117","0124",
                "0202","0209","0215","0223",
                "0302","0309","0316","0324",
                "0401","0408","0414","0423","0430",
                "0507","0514","0522","0530",
                "0605","0613","0621","0628",
                "0705","0712","0720","0727",
                "0803","0811","0819","0825",
                "0902","0910","0917","0924",
                "1001","1010","1016","1023","1031",
                "1108","1115","1122","1130",
                "1208","1214","1221","1230"
            ]
        ],

        /**
         * For each year, set first stamp. (2016 ~ 2020)
         * 0 : New Moon
         * 1 : First Quarter
         * 2 : Full Moon
         * 3 : Last Quarter
         * @type {number[]}
         */
        MOON_PHASE_FIRST_ANGLE = [
            3, 1, 2, 0, 1
        ],

        /**
         * Total days of each month. ( January ~ December )
         * @type {number[]}
         */
        YEAR_DAY = [
            31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
        ],

        /**
         * Total days of each month as leap year. ( January ~ December )
         * @type {number[]}
         */
        LEAP_YEAR_DAY = [
            31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
        ],
        MINIMUM_DAY_DIFFERENCE = 32,
        currentStampNumber,
        startStamp,
        lastStamp,
        year,
        month,
        date,
        fullDate,
        angle;

    /**
     * Returns whether a leap year.
     * @private
     * @param {number} year
     * @return {boolean} true if the year is a leap year.
     */
    function isLeapYear(year) {
        var isLeap = tizen.time.isLeapYear(year);

        return isLeap;
    }

    /**
     * Calculates rotating angle for moon phase.
     * @private
     */
    function calcAngle() {
        var denomiStamp = lastStamp - startStamp + 1,
            numerStamp = fullDate - startStamp + 1,
            startSubMonth = parseInt(startStamp.substring(0, 2), 10),
            startSubDay = parseInt(startStamp.substring(2, 4), 10),
            lastSubDay = parseInt(lastStamp.substring(2, 4), 10),
            todaySubDay = parseInt(fullDate.substring(2, 4), 10),
            startYearMonthDay;

        // If current year is leap year.
        if( isLeapYear(year) ) {
            startYearMonthDay = LEAP_YEAR_DAY[startSubMonth-1];
        }
        else {
            startYearMonthDay = YEAR_DAY[startSubMonth-1];
        }

        if(denomiStamp > MINIMUM_DAY_DIFFERENCE || denomiStamp < 0) {
            denomiStamp =  (startYearMonthDay - startSubDay) + 1 + lastSubDay;
        }

        if(numerStamp > MINIMUM_DAY_DIFFERENCE || numerStamp < 0) {
            numerStamp = (startYearMonthDay - startSubDay) + 1 + todaySubDay;
        }

        if(startStamp === lastStamp) {
            angle = 45 * currentStampNumber;
        }
        else {
            angle = (numerStamp / denomiStamp * 45) + 45 * currentStampNumber;
        }
    }

    /**
     * Sets current stamp.
     * @private
     */
    function setCurrentStamp() {
        var i;

        startStamp = lastStamp = "NULL";

        // find stamp using current date.
        for( i=0; i<MOON_PHASE_DATA[year].length; i++ ) {

            // If stamp date equal to current date, set current stamp to that stamp.
            if(MOON_PHASE_DATA[year][i] === fullDate ) {
                startStamp = lastStamp = MOON_PHASE_DATA[year][i];
                currentStampNumber = (MOON_PHASE_FIRST_ANGLE[year] + i) % 4;
                break;
            }

            // If that's not it, find the median stamp.
            else if(MOON_PHASE_DATA[year][i] > fullDate) {

                // [year][i-1] ~ [year][i]
                startStamp = MOON_PHASE_DATA[year][i-1];
                lastStamp = MOON_PHASE_DATA[year][i];

                // If today is smaller than the first stamp,
                // set startStamp is the last stamp of last year.
                if(i === 0) {
                    startStamp = MOON_PHASE_DATA[year-1][MOON_PHASE_DATA[year-1].length - 1];
                    lastStamp = MOON_PHASE_DATA[year][i];
                }

                currentStampNumber = (MOON_PHASE_FIRST_ANGLE[year] + i + 3) % 4;
                break;
            }
        }

        // There is no stamp in current year, set the first stamp of next year.
        if(startStamp === "NULL" && lastStamp === "NULL")
        {
            startStamp = MOON_PHASE_DATA[year][MOON_PHASE_DATA[year].length - 1];
            lastStamp = MOON_PHASE_DATA[year+1][0];
            currentStampNumber = (MOON_PHASE_FIRST_ANGLE[year] + i + 3) % 4;
        }
    }

    /**
     * Receives input date information, then, sets current stamp.
     * Calculates angle as current stamp.
     * @public
     * @param {object} dateInformation
     * @return {number} angle
     */
    moonphase.inputDate = function inputDate(dateInformation){
        // Handles exception status.
        // This sample handles specific years. (2016 ~ 2020)
        if(dateInformation.year > 2020 || dateInformation.year < 2016) {
            return 0;
        }

        year = dateInformation.year - 2016;
        month = dateInformation.month;
        date = dateInformation.date;
        fullDate = month + date;

        setCurrentStamp();
        calcAngle();

        return angle;
    };

    return moonphase;
}());